@@ -63,7 +63,12 @@ def init(flavour, template, checkout):
               help='If specified, allows the installation of alpha releases')
 @click.option('--lock/--skip-lock', default=True, is_flag=True,
               help='Lock dependencies or avoid this step')
-def install(pre, lock):
+@click.option(
+    '--production/--development', '-p/-d', default=True, is_flag=True,
+    help='Production mode copies statics/assets. Development mode symlinks'
+         ' statics/assets.'
+)
+def install(pre, lock, production):
     """Installs the  project locally.
 
     Installs dependencies, creates instance directory,
@@ -72,7 +77,11 @@ def install(pre, lock):
     """
     cli_config = CLIConfig()
     commands = LocalCommands(cli_config)
-    commands.install(pre=pre, lock=lock)
+    commands.install(
+        pre=pre,
+        lock=lock,
+        flask_env='production' if production else 'development'
+    )
 
 
 @cli.command()
@@ -95,9 +104,7 @@ def services(force):
               help='Force recreation of db tables, ES indices, queues...')
 @click.option('--install-js/--no-install-js', default=True, is_flag=True,
               help="(re-)Install JS dependencies, defaults to True")
-@click.option('--stop', default=False, is_flag=True,
-              help='Stop containers')
-def containerize(pre, force, install_js, stop):
+def containerize(pre, force, install_js):
     """Setup and run all containers (docker-compose.full.yml).
 
     Think of it as a production compilation build + running.
@@ -108,7 +115,7 @@ def containerize(pre, force, install_js, stop):
         DockerHelper(cli_config.get_project_shortname(), local=False)
     )
 
-    commands.containerize(pre=pre, force=force, install=install_js, stop=stop)
+    commands.containerize(pre=pre, force=force, install=install_js)
 
 
 @cli.command()
@@ -123,44 +130,127 @@ def demo(local):
 
 @cli.command()
 @click.option('--host', '-h',  default='127.0.0.1',
-              help='bind address')
+              help='The interface to bind to.')
 @click.option('--port', '-p',  default=5000,
-              help='bind port')
-def run(host, port):
+              help='The port to bind to.')
+@click.option('--debug/--no-debug', '-d/',  default=True, is_flag=True,
+              help='Enable/disable debug mode including auto-reloading '
+                   '(default: enabled).')
+def run(host, port, debug):
     """Starts the local development server.
 
     NOTE: this only makes sense locally so no --local option
     """
     cli_config = CLIConfig()
     commands = LocalCommands(cli_config)
-    commands.run(host=host, port=str(port))
+    commands.run(host=host, port=str(port), debug=debug)
 
 
-@cli.command()
-@click.option('--install-js/--no-install-js', default=False, is_flag=True,
-              help='(re-)Install JS dependencies, defaults to False')
-def update(install_js):
+@cli.group()
+def assets():
+    """Statics and assets management commands."""
+
+
+@assets.command()
+@click.option('--force', '-f', default=False, is_flag=True,
+              help='Force the full recreation the assets and statics.')
+@click.option(
+    '--production/--development', '-p/-d', default=True, is_flag=True,
+    help='Production mode copies files. Development mode symlinks files.'
+)
+def update(force, production):
     """Updates the current application static/assets files."""
     cli_config = CLIConfig()
     commands = LocalCommands(cli_config)
-    commands.update_statics_and_assets(install=install_js)
+    commands.update_statics_and_assets(
+        force=force,
+        flask_env='production' if production else 'development'
+    )
+
+
+@assets.command()
+def watch():
+    """Watch assets files for changes and rebuild."""
+    cli_config = CLIConfig()
+    commands = LocalCommands(cli_config)
+    commands.watch_assets()
+
+
+@assets.command('install-module')
+@click.argument('path', type=click.Path(exists=True))
+def install_module(path):
+    """Install and link a React module."""
+    cli_config = CLIConfig()
+    commands = LocalCommands(cli_config)
+    commands.link_js_module(path)
+
+
+@assets.command('watch-module')
+@click.option('--link', '-l', default=False, is_flag=True,
+              help='Link the module.')
+@click.argument('path', type=click.Path(exists=True))
+def watch_module(path, link):
+    """Watch a React module."""
+    cli_config = CLIConfig()
+    commands = LocalCommands(cli_config)
+    commands.watch_js_module(path, link=link)
 
 
 @cli.command()
-@click.option('--local/--containers', default=True, is_flag=True,
-              help='Which environment to build, it defaults to local')
 @click.option('-v', '--verbose', default=False, is_flag=True, required=False,
               help='Verbose mode will show all logs in the console.')
-def destroy(local, verbose):
+def destroy(verbose):
     """Removes all associated resources (containers, images, volumes)."""
     cli_config = CLIConfig()
-    commands = Commands(cli_config, local)
+    commands = Commands(cli_config, False)
     commands.destroy()
 
 
+@cli.command()
+def stop():
+    """Stops containers."""
+    cli_config = CLIConfig()
+    commands = Commands(cli_config, False)
+    commands.stop()
+
+
 @cli.command()
 @click.option('-v', '--verbose', default=False, is_flag=True, required=False,
               help='Verbose mode will show all logs in the console.')
 def upgrade(verbose):
     """Upgrades the current application to the specified newer version."""
     click.secho('TODO: Implement upgrade command', fg='red')
+
+
+@cli.command()
+def shell():
+    """Shell command."""
+    cli_config = CLIConfig()
+    commands = LocalCommands(cli_config)
+    commands.shell()
+
+
+@cli.command()
+@click.option(
+    '--debug/--no-debug', '-d/', default=False, is_flag=True,
+    help='Enable Flask development mode (default: disabled).'
+)
+def pyshell(debug):
+    """Python shell command."""
+    cli_config = CLIConfig()
+    commands = LocalCommands(cli_config)
+    commands.pyshell(debug=debug)
+
+
+@cli.group()
+def ext():
+    """Commands for development."""
+
+
+@ext.command('module-install')
+@click.argument("modules", nargs=-1, type=str)
+def module_install(modules):
+    """Install a Python module in development mode."""
+    cli_config = CLIConfig()
+    commands = Commands(cli_config, True)
+    commands.install_modules(modules)
