@@ -13,11 +13,14 @@ import signal
 import subprocess
 import time
 from distutils import dir_util
+from pathlib import Path
 
 import click
+from pynpm import NPMPackage
 
 from . import filesystem
 from .docker_helper import DockerHelper
+from .env import env
 
 
 class Commands(object):
@@ -61,6 +64,21 @@ class LocalCommands(object):
             command += ['--skip-lock']
         subprocess.run(command, check=True)
 
+    def install_modules(self, modules):
+        """Install modules."""
+        if len(modules) < 1:
+            raise click.UsageError("You must specify at least one module.")
+
+        cmd = ['pipenv', 'run', 'pip', 'install']
+        for m in modules:
+            cmd.append('-e')
+            cmd.append(m)
+
+        try:
+            subprocess.run(cmd, check=True)
+        except subprocess.CalledProcessError:
+            click.secho('You must specify a valid path.', fg='red')
+
     def _update_instance_path(self):
         """Update path to instance in config."""
         path = subprocess.run(
@@ -80,7 +98,7 @@ class LocalCommands(object):
     def _symlink_assets_templates(self, files_to_link):
         """Symlink the assets folder."""
         assets = 'assets'
-        click.secho('Symlinking {}/...'.format(assets), fg='green')
+        click.secho('Symlinking {}...'.format(assets), fg='green')
 
         instance_path = str(self.cli_config.get_instance_path()) + '/'
         project_dir = str(self.cli_config.get_project_dir())
@@ -110,37 +128,92 @@ class LocalCommands(object):
         copied_files = dir_util.copy_tree(src_dir, dst_dir)
         return copied_files
 
-    def update_statics_and_assets(self, install):
-        """High-level command to update scss/js/images... files."""
-        click.secho('Collecting statics and assets...', fg='green')
-        # Collect into statics/ folder
-        command = ['pipenv', 'run', 'invenio', 'collect', '--verbose']
-        subprocess.run(command, check=True)
-        # Collect into assets/ folder
-        command = ['pipenv', 'run', 'invenio', 'webpack', 'create']
-        subprocess.run(command, check=True)
+    def update_statics_and_assets(self, force, flask_env='production'):
+        """High-level command to update less/js/images/... files."""
+        # Commands
+        prefix = ['pipenv', 'run']
+        collect_cmd = prefix + ['invenio', 'collect', '--verbose']
+        clean_create_cmd = prefix + ['invenio', 'webpack', 'clean', 'create']
+        create_cmd = prefix + ['invenio', 'webpack', 'create']
+        install_cmd = prefix + ['invenio', 'webpack', 'install']
+        build_cmd = prefix + ['invenio', 'webpack', 'build']
 
-        if install:
-            click.secho('Installing js dependencies...', fg='green')
-            # Installs in assets/node_modules/
-            command = ['pipenv', 'run', 'invenio', 'webpack', 'install']
-            subprocess.run(command, check=True)
+        with env(FLASK_ENV=flask_env):
+            # Collect into statics/ and assets/ folder
+            click.secho('Collecting statics and assets...', fg='green')
+            subprocess.run(collect_cmd, check=True)
+            if force:
+                subprocess.run(clean_create_cmd, check=True)
 
-        copied_files = self._copy_statics_and_assets()
-        self._symlink_assets_templates(copied_files)
+                # Installs in assets/node_modules/
+                click.secho('Installing JS dependencies...', fg='green')
+                subprocess.run(install_cmd, check=True)
+            else:
+                subprocess.run(create_cmd, check=True)
 
-        click.secho('Building assets...', fg='green')
-        command = ['pipenv', 'run', 'invenio', 'webpack', 'build']
-        subprocess.run(command, check=True)
+            # Symlink the instance's statics and assets
+            copied_files = self._copy_statics_and_assets()
+            self._symlink_assets_templates(copied_files)
 
-    def install(self, pre, lock):
+            # Build project
+            click.secho('Building assets...', fg='green')
+            subprocess.run(build_cmd, check=True)
+
+    def watch_assets(self):
+        """High-level command to watch assets for changes."""
+        # Commands
+        prefix = ['pipenv', 'run']
+        watch_cmd = prefix + ['invenio', 'webpack', 'run', 'start']
+
+        with env(FLASK_ENV='development'):
+            # Collect into statics/ and assets/ folder
+            click.secho('Starting assets watching (press CTRL+C to stop)...',
+                        fg='green')
+            subprocess.run(watch_cmd, check=True)
+
+    @staticmethod
+    def _watch_js_module(pkg):
+        """Watch the JS module for changes."""
+        click.secho('Starting watching module...', fg='green')
+        pkg.run_script('watch')
+
+    @staticmethod
+    def _module_pkg(path):
+        """NPM package for the given path."""
+        return NPMPackage(Path(path) / 'package.json')
+
+    def _assets_pkg(self):
+        """NPM package for the instance's webpack project."""
+        return self._module_pkg(self.cli_config.get_instance_path() / 'assets')
+
+    def link_js_module(self, path):
+        """High-level command to install and build a JS module."""
+        module_pkg = self._module_pkg(path)
+        assets_pkg = self._assets_pkg()
+
+        # Create link to global folder
+        click.secho('Linking module...', fg='green')
+        module_pkg.run_script('link-dist')
+
+        # Link the global folder to the assets folder.
+        assets_pkg.link(module_pkg.package_json['name'])
+
+    def watch_js_module(self, path, link=True):
+        """High-level command to watch a JS module for changes."""
+        if link:
+            self.link_js_module(path)
+
+        click.secho('Starting watching module...', fg='green')
+        self._module_pkg(path).run_script('watch')
+
+    def install(self, pre, lock, flask_env='production'):
         """Local build."""
         self._install_py_dependencies(pre, lock)
         self._update_instance_path()
         self._symlink_project_file_or_folder('invenio.cfg')
         self._symlink_project_file_or_folder('templates')
         self._symlink_project_file_or_folder('app_data')
-        self.update_statics_and_assets(install=True)
+        self.update_statics_and_assets(force=True, flask_env=flask_env)
 
     def _ensure_containers_running(self):
         """Ensures containers are running."""
@@ -228,7 +301,7 @@ class LocalCommands(object):
         command = ['pipenv', 'run', 'invenio', 'rdm-records', 'demo']
         subprocess.run(command, check=True)
 
-    def run(self, host, port):
+    def run(self, host, port, debug):
         """Run development server and celery queue."""
         self._ensure_containers_running()
 
@@ -247,7 +320,8 @@ class LocalCommands(object):
 
         click.secho("Starting up local (development) server...", fg='green')
         run_env = os.environ.copy()
-        run_env['FLASK_ENV'] = 'development'
+        run_env['FLASK_ENV'] = 'development' if debug else 'production'
+        run_env['INVENIO_SITE_HOSTNAME'] = f"{host}:{port}"
         server = subprocess.Popen([
             'pipenv', 'run', 'invenio', 'run', '--cert',
             'docker/nginx/test.crt', '--key', 'docker/nginx/test.key',
@@ -259,23 +333,16 @@ class LocalCommands(object):
             fg='green')
         server.wait()
 
-    def destroy(self):
-        """Destroys a virtualenv (if created with pipenv) and containers."""
-        try:
-            subprocess.run(['pipenv', '--rm'], check=True)
-            click.secho('Virtual environment destroyed', fg='green')
-        except subprocess.CalledProcessError:
-            click.secho('The virtual environment was '
-                        'not removed as it was not '
-                        'created by pipenv', fg='red')
+    def shell(self):
+        """Start a shell in the virtual environment."""
+        command = ['pipenv', 'shell', ]
+        subprocess.run(command, check=True)
 
-        docker_helper = DockerHelper(
-            self.cli_config.get_project_shortname(),
-            local=True)
-
-        self.cli_config.update_services_setup(False)
-        docker_helper.destroy_containers()
-        click.secho('Destroyed containers...', fg='green')
+    def pyshell(self, debug=False):
+        """Start a Python shell."""
+        with env(FLASK_ENV='development' if debug else 'production'):
+            command = ['pipenv', 'run', 'invenio', 'shell']
+            subprocess.run(command, check=True)
 
 
 class ContainerizedCommands(object):
@@ -334,13 +401,8 @@ class ContainerizedCommands(object):
         if not locked:
             subprocess.run(command, check=True)
 
-    def containerize(self, pre, force, install, stop):
+    def containerize(self, pre, force, install):
         """Launch fully containerized application."""
-        if stop:
-            self.docker_helper.stop_containers()
-            click.secho("Stopping containers...", fg="green")
-            return
-
         self._lock_python_dependencies(pre)
 
         click.secho('Making sure containers are up...', fg='green')
@@ -413,6 +475,12 @@ class ContainerizedCommands(object):
         self.docker_helper.execute_cli_command(
             project_shortname, 'invenio rdm-records demo')
 
+    def stop(self):
+        """Stops containers."""
+        click.secho("Stopping containers...", fg="green")
+        self.docker_helper.stop_containers()
+        click.secho('Stopped containers', fg='green')
+
     def destroy(self):
         """Destroys the instance's virtualenv and containers."""
         try:
@@ -425,4 +493,4 @@ class ContainerizedCommands(object):
 
         self.docker_helper.destroy_containers()
         self.cli_config.update_services_setup(False)
-        click.secho('Destroyed containers...', fg='green')
+        click.secho('Destroyed containers', fg='green')
