@@ -57,11 +57,12 @@ class DockerHelper(object):
 
     def stop_containers(self):
         """Stop currently running containers."""
-        command = ['docker-compose',
-                   '--file', 'docker-compose.full.yml', 'stop']
-
-        if self.local:
-            command[2] = 'docker-compose.yml'
+        command = [
+            'docker-compose',
+            '--file',
+            'docker-compose.full.yml',
+            'stop'
+        ]
 
         subprocess.call(command)
 
@@ -69,8 +70,6 @@ class DockerHelper(object):
         """Stop and remove all containers, volumes and images."""
         command = ['docker-compose', '--file', 'docker-compose.full.yml',
                    'down', '--volumes']
-        if self.local:
-            command[2] = 'docker-compose.yml'
 
         subprocess.call(command)
 
