@@ -8,9 +8,12 @@
 
 """Module commands.py's tests."""
 import os
+import pathlib
+import subprocess
 from pathlib import Path
 from unittest.mock import Mock, call, patch
 
+import click
 import pytest
 
 from invenio_cli.helpers.commands import Commands, ContainerizedCommands, \
@@ -138,11 +141,12 @@ def test_localcommands_update_statics_and_assets(
 
     commands = LocalCommands(fake_cli_config)
 
-    commands.update_statics_and_assets(install=True)
+    commands.update_statics_and_assets(force=True)
 
     expected_calls = [
         call(['pipenv', 'run', 'invenio', 'collect', '--verbose'], check=True),
-        call(['pipenv', 'run', 'invenio', 'webpack', 'create'], check=True),
+        call(['pipenv', 'run', 'invenio', 'webpack', 'clean', 'create'],
+             check=True),
         call(['pipenv', 'run', 'invenio', 'webpack', 'install'], check=True),
         call(['pipenv', 'run', 'invenio', 'webpack', 'build'], check=True),
     ]
@@ -157,7 +161,7 @@ def test_localcommands_update_statics_and_assets(
     # Reset for install=False assertions
     patched_subprocess.run.reset_mock()
 
-    commands.update_statics_and_assets(install=False)
+    commands.update_statics_and_assets(force=False)
 
     expected_calls = [
         call(['pipenv', 'run', 'invenio', 'collect', '--verbose'], check=True),
@@ -167,6 +171,21 @@ def test_localcommands_update_statics_and_assets(
     assert patched_subprocess.run.mock_calls == expected_calls
 
 
+@patch('invenio_cli.helpers.commands.subprocess')
+@patch('invenio_cli.helpers.commands.time')
+@patch('invenio_cli.helpers.commands.DockerHelper')
+def test_localcommands_watch(
+        patched_docker_helper, patched_time, patched_subprocess,
+        fake_cli_config):
+
+    LocalCommands(fake_cli_config).watch_assets()
+
+    patched_subprocess.run.assert_called_with(
+        ['pipenv', 'run', 'invenio', 'webpack', 'run', 'start'],
+        check=True
+    )
+
+
 def test_localcommands_install():
     commands = LocalCommands(fake_cli_config)
     commands._install_py_dependencies = Mock()
@@ -187,7 +206,8 @@ def test_localcommands_install():
         commands._symlink_project_file_or_folder.mock_calls ==
         expected_symlink_calls
     )
-    commands.update_statics_and_assets.assert_called_with(install=True)
+    commands.update_statics_and_assets.assert_called_with(
+        force=True, flask_env='production')
 
 
 @patch('invenio_cli.helpers.commands.subprocess')
@@ -259,34 +279,6 @@ def test_localcommands_demo(
     )
 
 
-@patch('invenio_cli.helpers.commands.subprocess')
-@patch('invenio_cli.helpers.docker_helper.subprocess')
-def test_localcommands_destroy(
-        docker_helper_subprocess, patched_subprocess, fake_cli_config):
-    commands = LocalCommands(fake_cli_config)
-
-    # Needed to fake call to docker-compose --version but not call
-    # to docker-compose up
-    def fake_normalize_name(self, project_shortname):
-        return 'project_shortname'
-
-    with patch.object(DockerHelper, '_normalize_name', fake_normalize_name):
-        def docker_helper(project_shortname, local):
-            return DockerHelper('project-shortname', local=True)
-        patch_handle = patch(
-            'invenio_cli.helpers.commands.DockerHelper', docker_helper
-        )
-        patch_handle.start()
-        commands.destroy()
-        patch_handle.stop()
-
-    patched_subprocess.run.assert_called_with(['pipenv', '--rm'], check=True)
-    docker_helper_subprocess.call.assert_called_with([
-        'docker-compose', '--file', 'docker-compose.yml', 'down', '--volumes'
-    ])
-    assert fake_cli_config.services_setup is False
-
-
 @patch('invenio_cli.helpers.commands.subprocess')
 @patch('invenio_cli.helpers.commands.time')
 @patch('invenio_cli.helpers.commands.DockerHelper')
@@ -295,10 +287,13 @@ def test_localcommands_run(
         fake_cli_config):
     commands = LocalCommands(fake_cli_config)
 
-    commands.run(host='127.0.0.1', port='5000')
+    host = '127.0.0.1'
+    port = '5000'
+    commands.run(host=host, port=port, debug=True)
 
     run_env = os.environ.copy()
     run_env['FLASK_ENV'] = 'development'
+    run_env['INVENIO_SITE_HOSTNAME'] = f"{host}:{port}"
     expected_calls = [
         call([
             'pipenv', 'run', 'celery', '--app', 'invenio_app.celery', 'worker'
@@ -322,7 +317,7 @@ def test_containerizedcommands_containerize(
     commands = ContainerizedCommands(fake_cli_config, patched_docker_helper())
 
     # Case: pre=False, force=False, install=True
-    commands.containerize(pre=False, force=False, install=True, stop=False)
+    commands.containerize(pre=False, force=False, install=True)
 
     commands.docker_helper.start_containers.assert_called()
     expected_setup_calls = [
@@ -352,7 +347,7 @@ def test_containerizedcommands_containerize(
     # Case: pre=False, force=False, install=False
     commands.docker_helper.execute_cli_command.reset_mock()
 
-    commands.containerize(pre=False, force=False, install=False, stop=False)
+    commands.containerize(pre=False, force=False, install=False)
 
     assert (
         call('project-shortname', 'invenio webpack install --unsafe') not in
@@ -362,7 +357,7 @@ def test_containerizedcommands_containerize(
     # Case: pre=False, force=True, install=True
     commands.docker_helper.execute_cli_command.reset_mock()
 
-    commands.containerize(pre=False, force=True, install=True, stop=False)
+    commands.containerize(pre=False, force=True, install=True)
 
     expected_force_calls = [
         call(
@@ -386,16 +381,20 @@ def test_containerizedcommands_containerize(
     # Only the locking of the Pipfile. No need to test all combinations.
     commands.docker_helper.execute_cli_command.reset_mock()
 
-    commands.containerize(pre=True, force=True, install=True, stop=False)
+    commands.containerize(pre=True, force=True, install=True)
 
     assert commands.docker_helper.execute_cli_command.mock_calls == (
         expected_force_calls + expected_setup_calls
     )
 
-    # Case: pre=False, force=False, install=False, stop=True
-    # Testing the stop_containers
-    commands.docker_helper.stop_containers.reset_mock()
-    commands.containerize(pre=False, force=False, install=False, stop=True)
+
+@patch('invenio_cli.helpers.commands.DockerHelper')
+def test_containerizedcommands_stop(
+        patched_docker_helper, fake_cli_config):
+    commands = ContainerizedCommands(fake_cli_config, patched_docker_helper())
+
+    commands.stop()
+
     commands.docker_helper.stop_containers.assert_called()
 
 
@@ -507,3 +506,121 @@ def test_containerizedcommands_destroy(
     patched_subprocess.run.assert_called_with(['pipenv', '--rm'], check=True)
     commands.docker_helper.destroy_containers.assert_called()
     assert fake_cli_config.services_setup is False
+
+
+@patch('invenio_cli.helpers.commands.subprocess')
+@patch('invenio_cli.helpers.commands.time')
+@patch('invenio_cli.helpers.commands.DockerHelper')
+def test_localcommands_shell(
+        patched_docker_helper, patched_time, patched_subprocess,
+        fake_cli_config):
+    LocalCommands(fake_cli_config).shell()
+    patched_subprocess.run.assert_called_with(
+        ['pipenv', 'shell', ],
+        check=True
+    )
+
+
+@patch('invenio_cli.helpers.commands.subprocess')
+@patch('invenio_cli.helpers.commands.time')
+@patch('invenio_cli.helpers.commands.DockerHelper')
+def test_localcommands_pyshell(
+        patched_docker_helper, patched_time, patched_subprocess,
+        fake_cli_config):
+    LocalCommands(fake_cli_config).pyshell()
+    patched_subprocess.run.assert_called_with(
+        ['pipenv', 'run', 'invenio', 'shell'],
+        check=True
+    )
+
+
+@patch('invenio_cli.helpers.commands.subprocess')
+def test_localcommands_install_modules(
+        patched_subprocess, fake_cli_config):
+    commands = LocalCommands(fake_cli_config)
+
+    """1 module"""
+    modules = [pathlib.Path().absolute()]
+    commands.install_modules(modules)
+    patched_subprocess.run.assert_called_with(
+        ['pipenv', 'run', 'pip', 'install', '-e', modules[0]], check=True)
+
+    """2 modules"""
+    modules = [pathlib.Path().absolute(), pathlib.Path().absolute()]
+    commands.install_modules(modules)
+    patched_subprocess.run.assert_called_with(
+        ['pipenv', 'run', 'pip', 'install', '-e', modules[0],
+            '-e', modules[1]], check=True)
+
+    """No modules"""
+    modules = []
+    with pytest.raises(click.UsageError):
+        commands.install_modules(modules)
+        cmd = ['pipenv', 'run', 'pip', 'install', '-e', modules]
+        patched_subprocess.run(cmd, check=True)
+
+    """Empty String"""
+    """
+    modules = [" "]
+    with pytest.raises(click.UsageError):
+        commands.install_modules(modules)
+        cmd = ['invenio-cli', 'ext', 'module-install', modules[0]]
+        print("comando")
+        print(cmd)
+        patched_subprocess.run(cmd, check=True)
+    """
+
+    """Invalid Path"""
+    """
+    modules = ["~/test/"]
+    with pytest.raises(subprocess.CalledProcessError):
+        commands.install_modules(modules)
+        cmd = ['invenio-cli', 'ext', 'module-install', modules]
+        patched_subprocess.run(cmd, check=True)
+    """
+
+
+@pytest.fixture()
+def testpkg():
+    return str(Path(__file__).parent / 'testpkg')
+
+
+@patch('pynpm.package.run_npm')
+def test_localcommands_link_js_module(
+        patched_run_npm, testpkg, fake_cli_config):
+
+    LocalCommands(fake_cli_config).link_js_module(testpkg)
+
+    expected_calls = [
+        call(testpkg, 'run-script', npm_bin='npm', args=('link-dist',)),
+        call('instance_dir/assets', 'link', npm_bin='npm', args=('testpkg',))
+    ]
+
+    assert expected_calls == patched_run_npm.mock_calls
+
+
+@patch('pynpm.package.run_npm')
+def test_localcommands_watch_js_module(
+        patched_run_npm, testpkg, fake_cli_config):
+
+    LocalCommands(fake_cli_config).watch_js_module(
+        testpkg, link=False)
+
+    expected_calls = [
+        call(testpkg, 'run-script', npm_bin='npm', args=('watch',)),
+    ]
+    assert expected_calls == patched_run_npm.mock_calls
+
+
+@patch('pynpm.package.run_npm')
+def test_localcommands_watch_js_module_w_build(
+        patched_run_npm, testpkg, fake_cli_config):
+
+    LocalCommands(fake_cli_config).watch_js_module(testpkg, link=True)
+
+    expected_calls = [
+        call(testpkg, 'run-script', npm_bin='npm', args=('link-dist',)),
+        call('instance_dir/assets', 'link', npm_bin='npm', args=('testpkg',)),
+        call(testpkg, 'run-script', npm_bin='npm', args=('watch',)),
+    ]
+    assert expected_calls == patched_run_npm.mock_calls
